/* Interfaces stub resolver testing support.
   Copyright (C) 2016 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

#ifndef TST_RESOLV_AUX_H
#define TST_RESOLV_AUX_H

#include <inttypes.h>
#include <netdb.h>
#include <stdbool.h>
#include <stdio.h>
#include <sys/types.h>

struct response_context
{
  const unsigned char *query_buffer;
  size_t query_length;
  int server_index;
  bool tcp;
};

struct resolv_aux;
struct response_builder;

enum
  {
    max_servers = 3,
  };

struct redirect_server
{
  bool disable_tcp;
  bool disable_udp;
};

struct redirect_config
{
  void (*response_callback) (const struct response_context *,
                             struct response_builder *,
                             const char *qname,
                             uint16_t qclass, uint16_t qtype);
  struct redirect_server servers[max_servers];

  /* Search path entries.  The first entry serves as the default
     domain name as well.  */
  const char *search[7];

  /* Number of servers to activate in resolv.  0 means the default,
     max_servers.  */
  int nscount;

  /* If true, use a single thread to process all UDP queries.  This
     results in more predictable ordering of queries.  */
  bool single_thread_udp;
};

struct resolv_aux *resolv_redirect (struct redirect_config);
void resolv_redirect_finish (struct resolv_aux *);

enum response_section
  {
    section_question,
    section_answer,
    section_authoritative,
    section_additional
  };

struct response_flags
{
  unsigned rcode;
  bool tc;

  /* Initial section count values.  Can be used to artificially
     increase the counts, for malformed packet testing.*/
  unsigned short qdcount;
  unsigned short ancount;
  unsigned short nscount;
  unsigned short adcount;
};

void response_init (struct response_builder *, struct response_flags);
void response_section (struct response_builder *, enum response_section);
void response_add_question (struct response_builder *,
                            const char *name, uint16_t class, uint16_t type);
void response_open_record (struct response_builder *, const char *name,
                           uint16_t class, uint16_t type, uint32_t ttl);
void response_add_data (struct response_builder *, const void *, size_t);
void response_add_name (struct response_builder *, const char *name);
void response_close_record (struct response_builder *);
void response_drop (struct response_builder *);
void response_close (struct response_builder *);

/* Return the number of bytes in the response buffer written so
   far.  */
size_t response_length (const struct response_builder *);

/* Return a copy of the response buffer constructed so far.  Its
   length can be obtained with response_length.  */
unsigned char *response_buffer (const struct response_builder *);

/* Comparison of responses.  */
char *format_hostent (struct hostent *);
void check_hostent (struct hostent *, const char *expected);
char *format_addrinfo (struct addrinfo *, int ret);
void check_addrinfo (struct addrinfo *, int ret, const char *expected);
char *format_packet (const unsigned char *buffer, size_t length);
void check_packet (const unsigned char *buffer, size_t length,
                   const char *original_qname);

/* Wrappers for pthread mutex functions.  */
void resolv_xlock (pthread_mutex_t *);
void resolv_xunlock (pthread_mutex_t *);

/* Wrappers for other libc functions.  */
struct resolv_memstream
{
  FILE *out;
  char *buffer;
  size_t length;
};

void resolv_xopen_memstream (struct resolv_memstream *);
void resolv_xfclose_memstream (struct resolv_memstream *);

#endif
